@@ -42,15 +42,15 @@
 #include <sstream>
 #include <signal.h>
 #include <stdlib.h>
-#include "platform/os.h"
-#include "platform/util/StringUtils.h"
-#include "platform/threads/threads.h"
+#include <p8-platform/os.h>
+#include <p8-platform/util/StringUtils.h>
+#include <p8-platform/threads/threads.h>
 #if defined(HAVE_CURSES_API)
   #include "curses/CursesControl.h"
 #endif
 
 using namespace CEC;
-using namespace PLATFORM;
+using namespace P8PLATFORM;
 
 #include "cecloader.h"
 
@@ -73,7 +73,7 @@ bool                 g_cursesEnable(false);
 CCursesControl        g_cursesControl("1", "0");
 #endif
 
-class CReconnect : public PLATFORM::CThread
+class CReconnect : public P8PLATFORM::CThread
 {
 public:
   static CReconnect& Get(void)
@@ -420,7 +420,6 @@ bool ProcessCommandTX(ICECAdapter *parser, const std::string &command, std::stri
   if (command == "tx" || command == "txn")
   {
     std::string strvalue;
-    uint8_t ivalue;
     cec_command bytes = parser->CommandFromString(arguments.c_str());
 
     if (command == "txn")
